<?php

/**
 * The file that defines all functions which are product specific.
 */

if ( ! defined( 'ABSPATH' ) ) { 
    exit; // Exit if file accessed directly
}

// Adding meta field for to set product DivideBuy enabled or disabled.
add_action( 'add_meta_boxes', 'add_product_meta_boxes' );
function add_product_meta_boxes() {
    add_meta_box( 
        'woocommerce-dividebuy-products', 
        __( 'DivideBuy' ), 
        'is_product_dividebuy', 
        'product', 
        'side', 
        'default' 
    );
}

// Displaying product meta field in product details page at admin side.
function is_product_dividebuy($post) {
    $values = get_post_custom( $post->ID );
    $product_dividebuy_support = isset( $values['product_dividebuy_support'] ) ? esc_attr( $values['product_dividebuy_support'][0] ) : "";
    $product_dividebuy_taxclass = isset( $values['product_dividebuy_taxclass'] ) ? esc_attr( $values['product_dividebuy_taxclass'][0] ) : "";
    
    wp_nonce_field( 'dividebuy_product_nonce', 'meta_box_nonce' ); ?>
    <label for="product_dividebuy_support"><?php esc_html_e( 'DivideBuy', 'woocommerce'); ?></label>
    <select name="product_dividebuy_support" id="product_dividebuy_support">
        <option value="disabled" <?php selected( $product_dividebuy_support, 'disabled' ); ?>>Disabled</option>
        <option value="enabled" <?php selected( $product_dividebuy_support, 'enabled' ); ?>>Enabled</option>
    </select>
    <br/><br/>
    <label for="product_dividebuy_taxclass"><?php esc_html_e( 'Tax Class', 'woocommerce'); ?></label>
    <select name="product_dividebuy_taxclass" id="product_dividebuy_taxclass">
        <option value="" <?php selected( $product_dividebuy_taxclass, '' ); ?>>---Select---</option>
        <option value="0" <?php selected( $product_dividebuy_taxclass, '0' ); ?>>VAT Zero - 0%</option>
        <option value="5" <?php selected( $product_dividebuy_taxclass, '5' ); ?>>VAT Reduced - 5%</option>
        <option value="7" <?php selected( $product_dividebuy_taxclass, '7' ); ?>>VAT Reduced - 7%</option>
        <option value="20" <?php selected( $product_dividebuy_taxclass, '20' ); ?>>VAT Standard - 20%</option>
    </select>
    <?php
}

// Saving value of meta field.
add_action( 'save_post', 'meta_box_product_save' );
function meta_box_product_save( $post_id ) {
    // Bail if we're doing an auto save
    if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) return;
     
    // if our nonce isn't there, or we can't verify it, bail
    if( !isset( $_POST['meta_box_nonce'] ) || !wp_verify_nonce( $_POST['meta_box_nonce'], 'dividebuy_product_nonce' ) ) return;
     
    // if our current user can't edit this post, bail
    if( !current_user_can( 'edit_post' ) ) return;
    
    if( isset( $_POST['product_dividebuy_support'] ) ){
        update_post_meta( $post_id, 'product_dividebuy_support', $_POST['product_dividebuy_support'] );
    }
    
    if( isset( $_POST['product_dividebuy_taxclass'] ) ){
        update_post_meta( $post_id, 'product_dividebuy_taxclass', $_POST['product_dividebuy_taxclass'] );
    }
}

/**
 * Adding new column in product listing page which displays "Yes" or "No"
 * based on divided buy supported or not.
 *
 * This column is also sortable.
 */

// Adding custom column for dividebuy supported products.
add_filter( 'manage_edit-product_columns', 'show_dividebuy_column', 10, 1 );
function show_dividebuy_column($columns){

   	// Adding dividebuy column
   	$columns['dividebuy'] = __( 'DivideBuy');

   	return $columns;
}

// Displaying values for DivideBuy column.
add_action( 'manage_product_posts_custom_column', 'product_column_dividebuy', 10, 2 );
function product_column_dividebuy( $column, $postid ) {
    
    switch ( $column ) {
        case 'dividebuy':
            $divide_buy_support = get_post_meta( $postid, 'product_dividebuy_support', true );
            if( $divide_buy_support == "enabled" ){
                    echo '<mark class="dividebuy_enabled">Enabled</mark>';
            }
            if( $divide_buy_support == "disabled" ){
                    echo '<mark class="dividebuy_disabled">Disabled</mark>';
            }
        break;
    }
}

// Making dividibuy column sortable.
add_filter( 'manage_edit-product_sortable_columns', 'make_dividebuy_column_sortable' );
function make_dividebuy_column_sortable( $columns ) {
	
    $columns['dividebuy'] = 'dividebuy';
    
    return $columns;
}

/**
 * Displaying DivideBuy banner in product details page as per selected option by admin.
 */

global $enabled_dividebuy_logo, $dividebuy_logo_url, $dividebuy_logo, $dividebuy_logo_position;
$enabled_dividebuy_logo = get_option('wc_settings_tab_dividebuy_enable_logo');
$dividebuy_logo_url = get_option('wc_settings_tab_dividebuy_logo');

if( trim($dividebuy_logo_url)!='' ) {
    $dividebuy_logo = '<img src="'.$dividebuy_logo_url.'" id="dividebuylogo" alt="DivideBuy" class="buttonImage tooltip1 pull-right"/>';
} else {
    $dividebuy_logo = '<img src="' . plugins_url() . '/dividebuy/assets/images/dividebuy-logo.png' . '" class="buttonImage tooltip1" id="dividebuylogo" alt="DivideBuy" width="260px"/>';
}

/**
 * Check if option for DivideBuy logo is enable or not. If yes then moving ahead.
 */

if( $enabled_dividebuy_logo == "yes" ) {
        

    // DivideBuy banner below "Add to cart" Button.
    add_action( 'woocommerce_after_add_to_cart_form', 'dividebuy_image_after_add_to_cart' );
    function dividebuy_image_after_add_to_cart() {
        global $dividebuy_logo;
        $DivideBuy_Helper = new DivideBuy_Helper();
        $result_ip = $DivideBuy_Helper->allowip_check();
        $product_dividebuy_support = get_post_meta( get_the_ID(), 'product_dividebuy_support', true );
        $available_gateways = WC()->payment_gateways->get_available_payment_gateways();
        if( $product_dividebuy_support == "enabled"  && array_key_exists("dividebuy",$available_gateways) && $result_ip == true){ ?>
           
            <div class="dividebuy_logo_section tooltip-logo clearfix">
                <div class="tool-tip-postion-left">
                    <div id="dividebuy_image_after_cart" class="tooltip-width pull-right">
                        <span class="btn-span" style="<?php echo get_option('wc_settings_tab_dividebuy_logo_custom_css');?>">
                            <?php echo $dividebuy_logo;?>
                        </span>
                    </div>
                          
                    <?php
                    $_product = wc_get_product( get_the_ID() );
                    $product_price = $_product->get_price_including_tax();
                    ?>
                    <div id="tooltip1" class="product_page_instalments tooltip-db">
                        <div class="tooltip-db-1 dnone">
                            <div class="instalment-details">
                            </div>
                             <div id="instalment-loader"></div>
                            <div class="hover-instalment instalment-5">
                                <div class="option-text blue-text">Select option at checkout</div>
                                <div class="highlights-instalments-text">
                                    O% APR Representative example. Credit subject to status and minimum order value.
                                </div>
                                <div class="tooltip-logo-right"> 
                                    <img src="<?php echo plugins_url() . '/dividebuy/assets/images/dividebuy.png' ;?>" width="100px"/>
                                </div>
                                <div class="clearfix"></div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            <script>
            jQuery( document ).ready( function() {
                var url = '<?php echo plugin_dir_url( __FILE__ ) . 'public/dividebuy-instalments.php';?>';
                jQuery("#dividebuylogo").on( 'mouseover', function() {
                    showDevideByTP = true;
                    jQuery(".tooltip-db-1").removeClass("dnone");
                    jQuery(".instalment-details").html("");
                    var price = 0;
                    

                    <?php 
                         $atrritbute = get_post_meta(get_the_ID(),'_product_attributes',true); 
                        if(!isset($atrritbute)){
                    ?>
                        var price = '<?php echo $product_price;?>';
                    <?php
  
                        } else{

                    ?>
                        var get_price_html = jQuery('.woocommerce-variation-price .price ins .amount').text();
                        var symbol = jQuery('.woocommerce-variation-price .price ins .woocommerce-Price-currencySymbol').text();
                        if(get_price_html == '')
                        {
                            var get_price_html = jQuery('.woocommerce-variation-price .price .amount').text();    
                            var symbol = jQuery('.woocommerce-variation-price .price .woocommerce-Price-currencySymbol').text();
                        }
                        if( get_price_html != '')
                        {
                            var comma_remove = get_price_html.replace(",", "");

                            var price = parseInt(comma_remove.replace("£", ""));     
                        }
                        else
                        {
                            var price = '<?php echo $product_price;?>';         
                        }
                        
                    <?php

                        }
                    ?>    

                    get_instalments(price, url, showTooltip);
                });

                jQuery(document).on("mouseleave", "#product_dividebuy_banner", function(e) {
                    showDevideByTP = false;
                });
            });
            </script>
            <?php
        }
    }
}

// Adding fields for displaying custom field for bulk edit and quick edit.
add_action( 'bulk_edit_custom_box', 'bulk_quick_edit_custom_box', 10, 2 );
add_action( 'quick_edit_custom_box', 'bulk_quick_edit_custom_box', 10, 2 );

function bulk_quick_edit_custom_box( $column_name, $post_type ) {

    switch ($post_type){
        
        case 'product':
            
            switch ($column_name){
            
                case 'price':
                    ?><fieldset class="inline-edit-col-right">
                        <div class="inline-edit-col">
                            <div class="inline-edit-group wp-clearfix">
                                <label class="alignleft">
                                    <span class="title">DivideBuy</span>
                                    <select name="product_dividebuy_support" id="product_dividebuy_support">
                                        <option value="">— No Change —</option>
                                        <option value="disabled">Disabled</option>
                                        <option value="enabled">Enabled</option>
                                    </select>
                                </label>
                            </div>
                        </div>
                    </fieldset><?php
                    break;
            }
            
    }    
}

// Saving DivideBuy data in quick edit of product.
add_action( 'save_post', 'manage_wp_posts_be_qe_save_post', 10, 2 );
function manage_wp_posts_be_qe_save_post( $post_id, $post ) {
    // pointless if $_POST is empty (this happens on bulk edit)
    if ( !isset( $_POST ) ){
        return $post_id;
    }
         
    // verify quick edit nonce
    if ( isset( $_POST[ '_inline_edit' ] ) && ! wp_verify_nonce( $_POST[ '_inline_edit' ], 'inlineeditnonce' ) ){
        return $post_id;
    }
             
    // don't save for autosave
    if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ){
        return $post_id;
    }
         
    // dont save for revisions
    if ( isset( $post->post_type ) && $post->post_type == 'revision' ){
        return $post_id;
    }
         
    switch( $post->post_type ) {
     
        case 'product':
         
            /**
             * Because this action is run in several places, checking for the array key
             * keeps WordPress from editing data that wasn't in the form, i.e. if you had
             * this post meta on your "Quick Edit" but didn't have it on the "Edit Post" screen.
             */
            $field = 'product_dividebuy_support';
            if ( isset( $_POST[ $field ] ) && count($_POST[ $field ]) > 0 ){
                update_post_meta( $post_id, $field, $_POST[ $field ] );
            }
            break;
             
    }
     
}

// Saving bulk products data for DivideBuy enable or disable.
add_action( 'wp_ajax_manage_wp_posts_using_bulk_quick_save_bulk_edit', 'manage_wp_posts_using_bulk_quick_save_bulk_edit' );
function manage_wp_posts_using_bulk_quick_save_bulk_edit() {
    // we need the post IDs
    $post_ids = ( isset( $_POST[ 'post_ids' ] ) && count($_POST[ $field ]) > 0 ) ? $_POST[ 'post_ids' ] : NULL;
         
    // if we have post IDs
    if ( isset( $post_ids ) && is_array( $post_ids ) ) {
     
        // get the custom fields
        $field = 'product_dividebuy_support';
             
        // if it has a value, doesn't update if empty on bulk
        if ( isset( $_POST[ $field ] ) && count($_POST[ $field ]) > 0 ) {

            // update for each post ID
            foreach( $post_ids as $post_id ) {
                update_post_meta( $post_id, $field, $_POST[ $field ] );
            }

        }
         
    }
     
}